-- --------------------------------------------------------------------------------
-- > lumBags (Kreoss @ Quel'Thalas EU) <
-- --------------------------------------------------------------------------------
-- Credits: Cargor for cargBags, Mono for m_Bags and Luzzifus for cargBags_Nivaya
-- --------------------------------------------------------------------------------

-- --------------------------------------------------------------------------------
-- Media // Config // Vars
-- --------------------------------------------------------------------------------

local border = "Interface\\ChatFrame\\ChatFrameBackground"
local font = "Interface\\AddOns\\SharedMedia\\fonts\\font.ttf"

local fontoutline = "OUTLINEMONOCHROME"
local fontsize = 8 -- Text font size
local scale = 1 -- Bags Scale

local bank_segmentation = false -- Separete bank into some categories

-- Vars

local cColor = RAID_CLASS_COLORS[select(2, UnitClass("player"))] -- Class Color

local filter = {}
_,_,_,filter.Consumables,filter.Glyphs,filter.Trades,_,_,_,filter.Gems,filter.Misc,filter.Quest = GetAuctionItemClasses() -- Item Filters

local HiddenBags = {} -- Bags being Hidden if using Segmentation

local Profit, Spent, OldMoney = 0, 0, 0 -- Gold Tooltip
if lumBagsData == nil then lumBagsData = {} end
local myRealm = GetRealmName()
local myPlayerName = UnitName("player")

-- ------------------------------------------------------------------------
-- Layout
-- ------------------------------------------------------------------------

local lumBags = cargBags:NewImplementation("lumBags")
lumBags:RegisterBlizzard()
local lumBags = cargBags:GetImplementation("lumBags")

-- It begins
function lumBags:OnInit()

	OldMoney = GetMoney() -- Save Initial Gold
	if lumBagsData.Gold == nil then lumBagsData.Gold = {} end
	if lumBagsData.Gold[myRealm] == nil then lumBagsData.Gold[myRealm] = {} end
	lumBagsData.Gold[myRealm][myPlayerName] = OldMoney

	local onlyBags 		 	= function(item) return item.bagID >= 0 and item.bagID <= 4 end -- The Bags
	local onlyKeyring     	= function(item) return item.bagID == -2 end -- Keyring keys
	local onlyBank 		  	= function(item) return item.bagID == -1 or item.bagID >= 5 and item.bagID <= 11 end -- Only the Bank bags
	local onlyBankFiltered	= function(item) return (item.bagID == -1 or item.bagID >= 5 and item.bagID <= 11) and item.type ~= filter.Trades and item.type ~= filter.Consumables and item.type ~= filter.Gems end -- Bank bags minus filtered stuff 
	local hideEmpty 		= function(item) return item.texture ~= nil end -- Hide Empty slots
	local onlyTrade 	  	= function(item) return item.type and (item.type == filter.Trades or item.type == filter.Gems) end -- Trade Good items
	local onlyConsumables 	= function(item) return item.type and (item.type == filter.Consumables) end -- Consumable items

	-- Fetches our container classes
	local MyContainer = lumBags:GetContainerClass()

	-- Bags
	main = MyContainer:New("Bags", {
			Columns = 9,
			Scale = scale,
			Bags = "backpack+bags",
			Movable = true,
			Title = "Inventory",
		})
		main:SetFilter(onlyBags, true)
		main:SetPoint("RIGHT", -71, 103)
						
	-- Keyring	
	keys = MyContainer:New("Keyring", {
			Columns = 10,
			Scale = main.Settings.Scale,
			Bags = "backpack+bags",
			Title = "Keyring",
		})
		keys:SetFilter(onlyKeyring, true)
		keys:SetFilter(hideEmpty , true)
		keys:Hide()
		
	-- Bank
	bank = MyContainer:New("Bank", {
			Columns = 12,
			Scale = main.Settings.Scale,
			Bags = "bankframe+bank",
			Title = "Bank",
		})
		if bank_segmentation then
			bank:SetFilter(onlyBankFiltered, true)
		else
			bank:SetFilter(onlyBank, true)
		end
		bank:SetPoint("BOTTOMRIGHT", main, "BOTTOMLEFT", -12, 0)
		bank:Hide()
		
	if bank_segmentation then
		-- Bank Trade Goods
		trade = MyContainer:New("Bank_Trade_Goods", {
				Columns = 12,
				Scale = main.Settings.Scale,
				Bags = "bankframe+bank",
				Title = "Bank Trade Goods",
			})
			trade:SetFilter(onlyTrade, true)
			trade:Hide()
			
		-- Bank Consumables
		consumables = MyContainer:New("Bank_Consumables", {
				Columns = 12,
				Scale = main.Settings.Scale,
				Bags = "bankframe+bank",
				Title = "Bank Consumables",
			})
			consumables:SetFilter(onlyConsumables, true)
			consumables:Hide()
	end
end

local MyContainer = lumBags:GetContainerClass()
local MyButton = lumBags:GetItemButtonClass()
local BagButton = lumBags:GetClass("BagButton", true, "BagButton")


function lumBags:ShowBags(...) for i = 1, select("#", ...) do local bag = select(i, ...) if not HiddenBags["cargBagslumBags"..bag.name] then bag:Show() end end end
function lumBags:HideBags(...) for i = 1, select("#", ...) do select(i, ...):Hide() end end

-- Bank Bags Opened
function lumBags:OnBankOpened()
	
	bank:Show()
	main:Show()
	if bank_segmentation then
		lumBags:ShowBags(trade, consumables)
	end
end

-- Bank Bags Closed
function lumBags:OnBankClosed()

	bank:Hide()
	if bank_segmentation then
		lumBags:HideBags(trade, consumables)
	end
end

-- Update Anchors
function lumBags:UpdateAnchors()

	--keys:SetPoint("BOTTOMLEFT", main, "TOPLEFT", 0, 10)
	
	if bank_segmentation then
		trade:SetPoint("BOTTOMLEFT", bank, "TOPLEFT", 0, 10)
		if(HiddenBags["cargBagslumBagsBank_Trade_Goods"]) then -- If Trade Goods bag is hidden
			consumables:SetPoint("BOTTOMLEFT", bank, "TOPLEFT", 0, 10)
		else
			consumables:SetPoint("BOTTOMLEFT", trade, "TOPLEFT", 0, 10)
		end
	end
end

-- Button Skinning
function MyButton:OnCreate()

self:SetBackdrop{
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = false, tileSize = 32, edgeSize = 1,
		insets = {left = 0, right = 0, top = 0, bottom = 0},
	}
	self:SetBackdropColor(0, 0, 0, .3)
	self:SetBackdropBorderColor(0, 0, 0, 1)

	local name = self:GetName()
	self.border = _G[name.."NormalTexture"]
	self.count  = _G[name.."Count"]
	self.glow	= _G[name.."IconQuestTexture"];
	
	-- Change Textures
	self:SetHighlightTexture("Interface\\AddOns\\lumBags\\media\\hover")
	-- self:SetNormalTexture("Interface\\AddOns\\lumBags\\media\\button")
	
	-- Fix the item count
	self.count:ClearAllPoints()
	self.count:SetFont(font, fontsize, fontoutline);
	self.count:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -2, 4)
	self.count:SetDrawLayer("OVERLAY")
	
	-- Resize Normal Texture
	self.border:ClearAllPoints()
	self.border:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
	self.border:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)
	
	-- Re-position the Rarity Glow
	self.glow:ClearAllPoints();
	self.glow:SetPoint("TOPLEFT", self, "TOPLEFT", -1, 1);
	self.glow:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 1, -1);
	
	
	-- Create Button background
	self.bg = CreateFrame("Frame", nil, self)
    self.bg:SetAllPoints(self)
	self.bg:SetPoint("TOPLEFT", self, "TOPLEFT", 0, 0)
    self.bg:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", 0, 0)
	self.bg:SetFrameLevel(0)
	local bckg = self.bg:CreateTexture(nil,"BACKGROUND",-8)
	bckg:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
    bckg:SetAllPoints(self)
	bckg:SetVertexColor(0,0,0,0)
end

function MyButton:OnUpdate()
	-- Color by bag type
	local bagType = (select(2, GetContainerNumFreeSlots(self.bagID)));
	-- Profession bags
	if(bagType and bagType > 4) then
		self.Border:SetVertexColor(0.1, 0.65, 0.1, 1)
	-- Normal bags
	else
		self.Border:SetVertexColor(0,0,0,0)
	end
end

function BagButton:OnCreate()
	self:GetCheckedTexture():SetVertexColor(0.3, 0.9, 0.9, 0.5)
end

function MyContainer:OnContentsChanged()
	
	local isEmpty = true
	local name = self:GetName()
	self:SortButtons("bagSlot")
	local width, height = self:LayoutButtons("grid", self.Settings.Columns, 4, 2, -22)
	local UpdateDimensions = function()
		if name == "cargBagslumBagsBags" or name == "cargBagslumBagsBank" then
			if(self.bagBar and self.bagBar:IsShown()) then
				self:SetSize(width - 39, height+10)
			else
				self:SetSize(width - 39, height-40)
			end
		else
			self:SetSize(width -39, height)
		end
	end
	if name ~= "cargBagslumBagsKeyring" then -- Ignore Keyring
		for i, button in pairs(self.buttons) do
			local item = lumBags:GetItemInfo(button.bagID, button.slotID)
			if item.link then
				isEmpty = false
				break
			end
		end
	end
	
	if (name ~= "cargBagslumBagsBags" or name ~= "cargBagslumBagsBank" or name ~= "cargBagslumBagsKeyring") then -- Don't hide Bags or Bank or Keyring
		if isEmpty then self:Hide() HiddenBags[name] = isEmpty else self:Show() HiddenBags[name] = false end -- Hides Frames that are empty and adds to Hidden Table
	end
	
	self.UpdateDimensions = UpdateDimensions
	self:UpdateDimensions()
	lumBags:UpdateAnchors(self)
end

-- OnCreate each frame do stuff
function MyContainer:OnCreate(name, settings)

	self.Settings = settings
	
	self:EnableMouse(true)

	self:SetBackdrop{
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
		tile = false, tileSize = 32, edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	}
	self:SetBackdropColor(0, 0, 0, 0)
	self:SetBackdropBorderColor(0, 0, 0, 0)
	

	self:SetParent(settings.Parent or lumBags)
	self:SetFrameStrata("HIGH")

	settings.Columns = settings.Columns or 10
	self:SetScale(settings.Scale or 1)
	
	-- Top Info Frame
	local topInfo = CreateFrame("Button", nil, self)
	topInfo:SetPoint("TOPLEFT", 2, -2)
	topInfo:SetWidth(50)
	--topInfo:SetPoint("TOPRIGHT", -100, 0)
	topInfo:SetHeight(20)
	
	-- Create a title to the Frames
	-- local title = self:CreateFontString(nil, "OVERLAY")
	-- title:SetFont(font, fontsize, fontoutline)
	-- title:SetPoint("LEFT", topInfo, "LEFT", -1, 0)
	--title:SetText("|cFF"..(format("%02x%02x%02x", 255, 255, 255))..self.Settings.Title.."|r")
		
	-- Creating infoFrame which serves as a basic bar for information and extra buttons
	local infoFrame = CreateFrame("Button", nil, self)
	infoFrame:SetPoint("TOPLEFT", 110, -2)
	infoFrame:SetPoint("TOPRIGHT", -50, 0)
	infoFrame:SetHeight(18)
	
	-- Close button
	if name == "Bags" or name == "Bank" then
		local close = CreateFrame("Button", nil, self, "UIPanelCloseButton")
		close:SetPoint("TOPRIGHT", 1, -1)
		close:SetSize(22,22)
		close:SetScript("OnClick", function() if lumBags:AtBank() then CloseBankFrame() else CloseAllBags() end end)
		close:GetNormalTexture():SetDesaturated(1)
	end

	-- Plugin: Free Space Tag
	-- if name == "Bags" or name == "Bank" then
		-- local space = self:SpawnPlugin("TagDisplay", "[space:free/max]", infoFrame)
		-- space:SetFont(font, fontsize, fontoutline)
		-- space:SetShadowOffset(0,0)
		-- space:SetPoint("LEFT", infoFrame, "LEFT")
		-- space.bags = cargBags:ParseBags(settings.Bags)
	-- end
	

	-- Gold Update
	local GoldUpdate = function(self,event)

		local MyMoney = GetMoney()
		local Diff = MyMoney - OldMoney
		
		Spent = Spent - Diff			
		Profit = Profit + Diff
		
		lumBagsData.Gold[myRealm][myPlayerName] = MyMoney
		
		OldMoney = MyMoney
	end
	
	local formatMoney = function(money)
		local g = floor(math.abs(money) / 10000)
		local s = mod(floor(math.abs(money) / 100), 100)
		local c = mod(floor(math.abs(money)), 100)
		if g ~= 0 then
			return format("%s".."|cffffd800g|r".." %s".."|cff989697s|r".." %s".."|cffa4633fc|r", g, s, c)
		elseif s ~= 0 then
			return format("%s".."|cff989697s|r".." %s".."|cffa4633fc|r", s, c)
		else
			return format("%s".."|cffa4633fc|r", c)
		end
	end
	
	local ResetGold = function()
		
		lumBagsData.Gold = {}
		lumBagsData.Gold[myRealm] = {}
		lumBagsData.Gold[myRealm][myPlayerName] = GetMoney()
		print("|cffe51616Gold Information Erased!|r")
	end
	
	-- Gold Frame Tooltip
	local goldTooltip = function()
		if not InCombatLockdown() then -- Don't Show in Combat
		GameTooltip:SetOwner(self, "ANCHOR_NONE")
		GameTooltip:SetPoint("BOTTOMLEFT", topInfo, "TOPLEFT", -11,10)
		GameTooltip:AddLine("Gold Info", cColor.r, cColor.g, cColor.b)
		if(Profit and Profit > Spent) then
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine("Session Profit", formatMoney(Profit), 0.61, 0.81, 0.05, 1, 1, 1)
		end
		if(Spent and Spent > 0) then
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine("Session Loss", formatMoney(Spent), 0.86, 0.04, 0.04, 1, 1, 1)
		end
		local goldList = lumBagsData.Gold[myRealm]
		if(goldList) then
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine("Server Gold", cColor.r,cColor.g,cColor.b)
			local totalGold = 0	
			for char, gold in pairs(goldList) do
				GameTooltip:AddDoubleLine(char, formatMoney(gold), 1, 1, 1, 1, 1, 1)
				totalGold = totalGold + gold
			end
			GameTooltip:AddDoubleLine(" ", "------------------", 0, 0, 0, 1, 1, 1)
			GameTooltip:AddDoubleLine("Total", formatMoney(totalGold), cColor.r,cColor.g,cColor.b, 1, 1, 1)
		end
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine("Right click to reset gold")
		GameTooltip:Show()
		end
	end
	
	-- Plugin: Currencies / Money Tag
	if name == "Bags" then
		local tagDisplay = self:SpawnPlugin("TagDisplay", "[money]", topInfo) -- Try to fix [currencies]
		tagDisplay:SetShadowOffset(0,0)
		tagDisplay:SetFont(font, fontsize, fontoutline)
		tagDisplay:SetPoint("LEFT", topInfo, "LEFT", 0, 0)
		tagDisplay.OnTagUpdate = GoldUpdate -- Gold PostUpdate
		
		-- Dummy Frame for Gold Tag
		local GoldFrame = CreateFrame("Button", nil, self)
		GoldFrame:SetFrameLevel(4)
		GoldFrame:SetAllPoints(tagDisplay)
		GoldFrame:SetScript("OnEnter", goldTooltip)
		GoldFrame:SetScript("OnLeave", function() GameTooltip:Hide() end)
		GoldFrame:RegisterForClicks("AnyUp")
		GoldFrame:SetScript("OnClick", function(self,button) if button == "RightButton" then ResetGold() end end)
	end
	
	-- Plugin: BagBar
	if name == "Bags" or name == "Bank" then
		local bagBar = self:SpawnPlugin("BagBar", settings.Bags)
		bagBar:SetScale(0.75)
		bagBar:Hide()
		self.bagBar = bagBar
		if(name == "Bank") then
			bagBar:SetSize(bagBar:LayoutButtons("grid", 7))
			bagBar:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 10, 15)
		else
			bagBar:SetSize(bagBar:LayoutButtons("grid", 4))
			bagBar:SetPoint("BOTTOMLEFT", self, "BOTTOMLEFT", 10, 15)
		end
		local bagToggle = CreateFrame("CheckButton", nil, self)
		bagToggle:SetWidth(36)
		bagToggle:SetHeight(30)
		bagToggle:SetPoint("TOPLEFT", infoFrame,"BOTTOMRIGHT",0,13)
		bagToggle:RegisterForClicks("LeftButtonUp")
		bagToggle:SetScript("OnClick", function()
		if(self.bagBar:IsShown()) then
				self.bagBar:Hide()
			else
				self.bagBar:Show()
			end
			self:UpdateDimensions()
		end)
		local bagToggleText = bagToggle:CreateFontString(nil, "OVERLAY")
		bagToggleText:SetPoint("CENTER", bagToggle, 0,10)
		bagToggleText:SetShadowOffset(0,0)
		bagToggleText:SetFont(font, fontsize, fontoutline)
		bagToggleText:SetText("Bags")
		bagToggleText:SetJustifyH("RIGHT")
		bagToggleText:SetTextColor(0.3,0.3,0.3,0.9)
	end
	
	-- Keyring button
	-- if name == "Bags" then
		-- local keysbutton = CreateFrame("CheckButton", nil, self.bagBar)
		-- keysbutton:SetSize(20,38)
		-- keysbutton:SetPoint("TOPLEFT", self.bagBar, "TOPRIGHT", 4, 0)
		-- keysbutton:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight", "ADD")
		-- keysbutton:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
		-- keysbutton.Texture = keysbutton:CreateTexture()
		-- keysbutton.Texture:SetTexture('Interface/Buttons/UI-Button-KeyRing')
		-- keysbutton.Texture:SetAllPoints(keysbutton)
		-- keysbutton.Texture:SetTexCoord(0, 0.5625, 0, 0.609375)
		-- keysbutton:SetScript("OnClick", function() 
			-- local keyring = lumBags:GetContainer("Keyring")
			-- if keyring:IsShown() then 
				-- keyring:Hide()
			-- else
				-- keyring:Show()
			-- end
		-- end)
	-- end

	-- Plugin: SearchBar
	local searchText = infoFrame:CreateFontString(nil, "OVERLAY")
	searchText:SetPoint("CENTER", infoFrame, "CENTER", -15,-1)
	searchText:SetFont(font, fontsize, fontoutline)
	searchText:SetText("Click to Search")
	searchText:SetTextColor(0.3,0.3,0.3,0.9)
	local search = self:SpawnPlugin("SearchBar",infoFrame)
	search.HighlightFunction = function(button, match)
		button:SetAlpha(match and 1 or 0.1)
	end
	-- search:SetScript("OnShow", function() -- Hide Title Text when Search is enabled
		-- title:Hide()
	-- end)
	-- search:SetScript("OnHide", function()
		-- title:Show()
	-- end)
		
	-- Make Bags Movable
	if(settings.Movable) then
		self:SetMovable(true)
		self:SetUserPlaced(true)
		self:RegisterForDrag("LeftButton")
		self:EnableMouse(true)
        self:SetScript("OnDragStart", function(self) if IsAltKeyDown() then self:StartMoving() end end)
        self:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
	end
end
